package kz.arta.demo;

import kz.arta.demo.asterisk.AsteriskEventListener;
import kz.arta.demo.utils.PropsUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import java.util.*;

/**
 * User: Aigul
 * Date: 11/9/16
 * Time: 5:48 PM
 *
 * Класс с таймерами которые запускаются при старте приложения
 */
@Startup
@Singleton
public class StartupTimer {
    private static final Logger LOGGER = LoggerFactory.getLogger(StartupTimer.class);

    private Timer gcTimer;

    //Актуализация скрытых полей сделки
    private Timer deployTimer;

    //Синхронизация календарей
    private Timer ldapSync;

    //Запуск соединения с астериском
    private Timer asteriskConnectionTimer;

    @PostConstruct
    public void run() {
        startCreateThread();
    }

    private void startCreateThread() {
        GregorianCalendar calendar = new GregorianCalendar();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 1);
        calendar.add(Calendar.DAY_OF_YEAR, 1);
        if (gcTimer != null) {
            gcTimer.cancel();
            gcTimer.purge();
        }
        gcTimer = new Timer();
        gcTimer.schedule(
                new TimerTask() {
                    @Override
                    public void run() {
                        createNotifications();
                    }
                }, calendar.getTime()
        );

        String enabled = PropsUtil.getProperty("asterisk.enabled");
        if (enabled != null && enabled.equals("true")) {
            asteriskConnectionTimer = new Timer();
            TimerTask hourlyTask = new TimerTask() {
                @Override
                public void run() {
                    String sync = PropsUtil.getProperty("ldap.sync.enabled");
                    AsteriskEventListener.closeConnections();
                    if (sync != null && sync.equals("true")) {
                        new AsteriskEventListener();
                        LOGGER.error("Asterisk restarted");
                    }
                }
            };

            asteriskConnectionTimer.schedule(hourlyTask, 0L, 1000 * 60 * 60);
        }

    }

    private void createNotifications() {
        startCreateThread();
    }

    @PreDestroy
    public void stop() {
        if (gcTimer != null) {
            gcTimer.cancel();
            gcTimer.purge();
            LOGGER.info("timer stopped");
        }
        if (deployTimer != null) {
            deployTimer.cancel();
            deployTimer.purge();
        }
        if (asteriskConnectionTimer != null) {
            asteriskConnectionTimer.cancel();
            asteriskConnectionTimer.purge();
            AsteriskEventListener.closeConnections();
        }
        if (ldapSync != null) {
            ldapSync.cancel();
            ldapSync.purge();
        }
    }

}
