package kz.arta.demo.socket;

import kz.arta.demo.objects.Contact;
import kz.arta.demo.utils.UserSearchService;
import kz.arta.synergy.forms.common.util.JsonUtil;
import org.atmosphere.jboss.as.websockets.WebSocket;
import org.atmosphere.jboss.as.websockets.servlet.WebSocketServlet;
import org.atmosphere.jboss.websockets.Frame;
import org.atmosphere.jboss.websockets.frame.TextFrame;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.AsyncContext;
import javax.servlet.AsyncEvent;
import javax.servlet.AsyncListener;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

@WebServlet(
        urlPatterns = {"/demo"},
        asyncSupported = true
)
public class Socket extends HttpServlet {
    private static final Logger LOGGER = LoggerFactory.getLogger(Socket.class);
    private static Map<String, List<AsyncContext>> asyncContexts = new ConcurrentHashMap<>();

    @Override
    public void destroy() {
        asyncContexts.clear();
    }

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        if ("text/event-stream".equals(request.getHeader("Accept"))) {
            request.setAttribute("org.apache.catalina.ASYNC_SUPPORTED", true);
            response.setContentType("text/event-stream");
            response.setHeader("Cache-Control", "no-cache");
            response.setHeader("X-Accel-Buffering", "no");
            response.setHeader("Connection", "keep-alive");
            response.setCharacterEncoding("UTF-8");
            response.getWriter().println("retry: 1000\n");

            final String userID = request.getParameter("userID");
            final AsyncContext ac = request.startAsync();
            ac.addListener(new AsyncListener() {
                @Override
                public void onComplete(AsyncEvent event) throws IOException {
                    if (asyncContexts.containsKey(userID)) {
                        asyncContexts.get(userID).remove(event.getAsyncContext());
                    }
                }

                @Override
                public void onError(AsyncEvent event) throws IOException {
                    if (asyncContexts.containsKey(userID)) {
                        asyncContexts.get(userID).remove(event.getAsyncContext());
                    }
                }

                @Override
                public void onStartAsync(AsyncEvent event) throws IOException {
                }

                @Override
                public void onTimeout(AsyncEvent event) throws IOException {
                    if (asyncContexts.containsKey(userID)) {
                        asyncContexts.get(userID).remove(event.getAsyncContext());
                    }
                }
            });
            if (asyncContexts != null && !asyncContexts.containsKey(userID)) {
                asyncContexts.put(userID, new ArrayList<AsyncContext>());
            }
            asyncContexts.get(userID).add(ac);
            sendMessage(userID, false, false);
        }
    }

    private static void sendMessage(String userID, Boolean isHangup, Boolean isDocID) {
        List<AsyncContext> acList = asyncContexts.get(userID);
        for (AsyncContext asyncContext : acList) {
            try {
                PrintWriter writer = asyncContext.getResponse().getWriter();
                writer.print("data: ");
                if (isHangup) {
                    writer.println(userID);
                    writer.println("HANGUP");
                } else if (isDocID) {
                    writer.println(userID);
                } else {
                    writer.println(userID);
                    writer.println("UP");
                }

                writer.println();
                writer.flush();
            } catch (Exception exc) {
                acList.remove(asyncContext);
                LOGGER.error(exc.getMessage(), exc);
            }
        }
    }

    public static void fireCallEvent(String from, String to) {
        if (to == null || to.trim().isEmpty() || to.equalsIgnoreCase("NULL")) {
            return;
        }
        if (from == null || from.trim().isEmpty() || from.equalsIgnoreCase("NULL")) {
            return;
        }
        String userID = UserSearchService.getUserByNumber(to);
        if (userID == null) {
            return;
        }
        String contact = UserSearchService.getUserByNumber(from);
        if (contact == null) {
            return;
        }

        try {
            sendMessage(contact, false, false);
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

    public static void hangUpEvent(String to) {
        if (to == null || to.trim().isEmpty() || to.equalsIgnoreCase("NULL")) {
            return;
        }
        String userID = UserSearchService.getUserByNumber(to);
        if (userID == null) {
            return;
        }

        try {
            sendMessage(userID, true, false);
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
    }
}
